/*
 * AbstractModel.java
 * Created in 18/04/2006
 */
package feec.ia725.cg.core;

import javax.media.opengl.GL;

/**
 * @author Lucas Shindi Shimo (lshimo@gmail.com)
 *
 */
public abstract class AbstractModel implements Model {
    private Vertex position;
    private Angle3d angle3d;
    private String code;
    private boolean alive;
    
    /**
     * @return Returns the angle3d.
     */
    public Angle3d getAngle3d() {
        return this.angle3d;
    }
    /**
     * @param angle3d The angle3d to set.
     */
    public void setAngle3d(Angle3d angle3d) {
        this.angle3d = angle3d;
    }
    /**
     * @return Returns the position.
     */
    public Vertex getPosition() {
        return this.position;
    }
    /**
     * @param position The position to set.
     */
    public void setPosition(Vertex position) {
    	double x = position.getX();
    	double y = position.getY();
    	double z = position.getZ();
        this.position = new Vertex(x, y, z);
    }
    /**
     * @return Returns the code.
     */
    public String getCode() {
        return this.code;
    }
    /**
     * @param code The code to set.
     */
    public void setCode(String code) {
        this.code = code;
    }
    
    /**
     * Realiza as transformaes geomtricas do objeto
     *
     */
    protected void doGeometricTransformations(GL gl) {
        
    	// altera a posio do objeto
    	if (position != null) {
    	
	        double x = this.getPosition().getX();
	        double y = this.getPosition().getY();
	        double z = this.getPosition().getZ();
	
	        // transladando
	        gl.glTranslated(x, y, z);
    	}
    	   	
    	// altera o ngulo do objeto
    	if (angle3d != null) {
    		
    		double theta  = angle3d.getTheta();
    		double phi  = angle3d.getPhi();

    		if (angle3d.getThetaAxle() != null && angle3d.getTheta() != 0) {
	    		// obtendo o eixo de rotao do angulo theta
    			Vertex axle = angle3d.getThetaAxle().normalizeVector();
    			
	    		double x,y,z;
	    		x = Math.abs(axle.getX());
	    		y = Math.abs(axle.getY());
	    		z = Math.abs(axle.getZ());
	
	    		// rotacionando de theta
	    		gl.glRotated(90 - theta, x,y,z);
    		}
    	
			// rotacionando de phi
			gl.glRotated(phi, 0,0,1);
			
			
    	}
    }
    /**
     * @return Returns the alive.
     */
    public boolean isAlive() {
        return this.alive;
    }
    /**
     * @param alive The alive to set.
     */
    public void setAlive(boolean alive) {
        this.alive = alive;
    }

}
