/*
 * Criado em Mar 21, 2006
 *
 */
package feec.ia725.cg.core;

/**
 * @author <a href="mailto:rmanoel@cpqd.com.br">Rafael H. Manoel</a>
 *
 * Representa uma caixa cobrindo os limites de um objeto 3D.  utilizada para
 * encontrar o centro de massa do modelo e para detectar colises.
 */
public class BoundingBox {

    private float maxX;
    
    private float  minX;
    
    private float minY;
    
    private float maxY;

    private float minZ;
    
    private float maxZ;

    /**
     * Inicializa o bounding box com valores default.
     *
     */
    public BoundingBox() {
        // atribuindo os valores mximo para as variveis que contero o menor valor
        minX = Float.MAX_VALUE;
        minY = Float.MAX_VALUE;
        minZ = Float.MAX_VALUE;
        
        // fazendo o inverso com as var de maior valor
        maxX = -Float.MAX_VALUE;
        maxY = -Float.MAX_VALUE;
        maxZ = -Float.MAX_VALUE;
    }
    
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#getCenterOfMass()
	 */
    public float[] getCenterOfMass() {
        float[] cm = new float[3];
        cm[0] = (maxX + minX) / 2.0f;
        cm[1] = (maxY + minY) / 2.0f;
        cm[2] = (maxZ + minZ) / 2.0f;
        return cm;
    }
    
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#normalize()
	 */
    public void normalize() {
        float maxCoord = getMaxCoord();
        maxX /= maxCoord;
        maxY /= maxCoord;
        maxZ /= maxCoord;

        minX /= maxCoord;
        minY /= maxCoord;
        minZ /= maxCoord;
    }
    
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#getMaxCoord()
	 */
    public float getMaxCoord() {
        if (maxX >= maxY && maxX >= maxZ) {
            return maxX;
        } else if (maxX < maxY && maxY >= maxZ) {
            return maxY;
        } else {
            return maxZ;
        }
    }
    
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#getMaxX()
	 */
    public float getMaxX() {
        return maxX;
    }
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#setMaxX(float)
	 */
    public void setMaxX(float x) {
        if (x > this.maxX) {
            this.maxX = x;
        }
    }
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#getMaxY()
	 */
    public float getMaxY() {
        return maxY;
    }
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#setMaxY(float)
	 */
    public void setMaxY(float y) {
        if (y > this.maxY) {
            this.maxY = y;
        }
    }
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#getMaxZ()
	 */
    public float getMaxZ() {
        return maxZ;
    }
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#setMaxZ(float)
	 */
    public void setMaxZ(float z) {
        if (z > this.maxZ) {
            this.maxZ = z;
        }
    }
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#getMinX()
	 */
    public float getMinX() {
        return minX;
    }
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#setMinX(float)
	 */
    public void setMinX(float x) {
        if (x < this.minX) {
            this.minX = x;
        }
    }
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#getMinY()
	 */
    public float getMinY() {
        return minY;
    }
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#setMinY(float)
	 */
    public void setMinY(float y) {
        if (y < this.minY) {
            this.minY = y;
        }
    }
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#getMinZ()
	 */
    public float getMinZ() {
        return minZ;
    }
    /* (non-Javadoc)
	 * @see feec.ia725.cg.core.BoundingBox#setMinZ(float)
	 */
    public void setMinZ(float z) {
        if (z < this.minZ) {
            this.minZ = z;
        }
    }

    /**
     * Escala o bounding box
     * @param scale fator de escala
     */
    public void scale(float scale) {
        maxX *= scale;
        maxY *= scale;
        maxZ *= scale;
        minX *= scale;
        minY *= scale;
        minZ *= scale;
    }
    
}
