/*
 * Color.java
 * Created in 05/04/2006
 */
package feec.ia725.cg.core;

/**
 * @author Lucas Shindi Shimo (lshimo@gmail.com)
 *
 */
public class Color extends Coordinate4d {
    /** Default Serial Version UID. */
    private static final long serialVersionUID = 1L;

    private static final double N = 255.0;

    /** The color white. In the default sRGB space. */
    public final static Color WHITE = new Color(255, 255, 255);
    /** The color light gray. In the default sRGB space. */
    public final static Color LIGHT_GRAY = new Color(192, 192, 192);
    /** The color gray. In the default sRGB space. */
    public final static Color GRAY = new Color(128, 128, 128);
        /** The color dark gray. In the default sRGB space. */
    public final static Color DARK_GRAY = new Color(64, 64, 64);
    /** The color black. In the default sRGB space. */
    public final static Color BLACK = new Color(0, 0, 0);
    /** The color red. In the default sRGB space. */
    public final static Color RED = new Color(255, 0, 0);
    /** The color pink. In the default sRGB space. */
    public final static Color PINK = new Color(255, 175, 175);
    /** The color orange. In the default sRGB space. */
    public final static Color ORANGE = new Color(255, 200, 0);
    /** The color yellow. In the default sRGB space. */
    public final static Color YELLOW = new Color(255, 255, 0);
    /** The color green. In the default sRGB space. */
    public final static Color GREEN = new Color(0, 255, 0);
    /** The color magenta. In the default sRGB space. */
    public final static Color MAGENTA = new Color(255, 0, 255);
    /** The color cyan. In the default sRGB space.  */
    public final static Color CYAN = new Color(0, 255, 255);
    /** The color blue. In the default sRGB space. */
    public final static Color BLUE = new Color(0, 0, 255);

    /////////////////////////
    //     CONSTRUCTORS    //
    /////////////////////////
    public Color(int r, int g, int b, int alpha) {
        super(r / N, g / N, b / N, alpha / N);
    }

    public Color(int r, int g, int b) {
        super(r / N, g / N, b / N, 1);
    }

    /** @see Coordinate4d#Coordinate4d(double, double, double, double) */
    public Color(double r, double g, double b, double alpha) {
        super(r, g, b, alpha);
    }

    /** @see Coordinate4d#Coordinate4d(Vertex, double) */
    public Color(Vertex vertex, double alpha) {
        super(vertex, alpha);
    }
    
    /** @see Coordinate4d#Coordinate4d(double, double, double) */
    public Color(double r, double g, double b) {
        super(r, g, b);
    }

    /** @see Coordinate4d#Coordinate4d(Vertex) */
    public Color(Vertex vertex) {
        super(vertex);
    }

    /////////////////////////////
    //    GETTERS / SETTERS    //
    /////////////////////////////
    /** @return Returns the b. */
    public int getB() {
        return (int) Math.round(this.getZ() * N);
    }
    /** @param b The b to set. */
    public void setB(int b) {
        super.setZ(b / N);
    }
    /** @return Returns the g. */
    public int getG() {
        return (int) Math.round(this.getY() * N);
    }
    /** @param g The g to set. */
    public void setG(int g) {
        super.setY(g / N);
    }
    /** @return Returns the r. */
    public int getR() {
        return (int) Math.round(this.getX() * N);
    }
    /** @param r The r to set. */
    public void setR(int r) {
        super.setX(r / N);
    }
    /** @return Returns the alpha. */
    public int getAlpha() {
        return (int) Math.round(this.getW() * N);
    }
    /** @param alpha The alpha to set. */
    public void setAlpha(int alpha) {
        super.setW(alpha / N);
    }
}
