package feec.ia725.cg.core;

public class Material {
	private Color ambient;
    private Color diffuse;
    private Color specular;
	private Color shininess;

    public static final Material BRASS = new Material(0.329412, 0.223529,
            0.027451, 0.780392, 0.568627, 0.113725, 0.992157, 0.941176,
            0.807843, 27.8974);

    public static final Material BRONZE = new Material(0.2125, 0.1275, 0.054,
            0.714, 0.4284, 0.18144, 0.393548, 0.271906, 0.166721, 25.6);

    public static final Material EMERALD = new Material(0.0215, 0.1745, 0.0215,
            0.55, 0.07568, 0.61424, 0.07568, 0.55, 0.633, 0.727811, 0.633,
            0.55, 76.8);

    public static final Material JADE = new Material(0.135, 0.2225, 0.1575,
            0.95, 0.54, 0.89, 0.63, 0.95, 0.316228, 0.316228, 0.316228, 0.95,
            12.8);

    public static final Material RUBY = new Material(0.1745, 0.01175, 0.01175,
            0.55, 0.61424, 0.04136, 0.04136, 0.55, 0.727811, 0.626959,
            0.626959, 0.55, 76.8);

    public static final Material BLACK_PLASTIC = new Material(0.0, 0.0, 0.0,
            1.0, 0.01, 0.01, 0.01, 1.0, 0.50, 0.50, 0.50, 1.0, 32);

    public static final Material GREEN_PLASTIC = new Material(0.0, 0.1, 0.0,
            1.0, 0.01, 0.51, 0.01, 1.0, 0.51, 0.51, 0.51, 1.0, 32);

    public static final Material PEARL = new Material(0.25, 0.20725, 0.20725,
            0.922, 1.0, 0.829, 0.829, 0.922, 0.296648, 0.296648, 0.296648,
            0.922, 11.264);

    public static final Material CHROME = new Material(0.25, 0.25, 0.25, 1.0,
            0.4, 0.4, 0.4, 1.0, 0.774597, 0.774597, 0.774597, 1.0, 76.8);
    /**
     * @param ar Ambient red
     * @param ag Ambient green
     * @param ab Ambient blue
     * @param dr Diffuse red
     * @param dg Diffuse green
     * @param db Diffuse blue
     * @param sr Specular red
     * @param sg Specular green
     * @param sb Specular blue
     * @param s Shininess
     */
    public Material(double ar, double ag, double ab, double dr, double dg,
            double db, double sr, double sg, double sb, double s) {
        this.ambient = new Color(ar, ag, ab);
        this.diffuse = new Color(dr, dg, db);
        this.specular = new Color(sr, sg, sb);
        this.shininess = new Color(s, 0, 0);
    }

    /**
     * @param ar Ambient red
     * @param ag Ambient green
     * @param ab Ambient blue
     * @param aa Ambient alpha
     * @param dr Diffuse red
     * @param dg Diffuse green
     * @param db Diffuse blue
     * @param da Diffuse alpha
     * @param sr Specular red
     * @param sg Specular green
     * @param sb Specular blue
     * @param sa Specular alpha
     * @param s Shininess
     */
    public Material(double ar, double ag, double ab, double aa, double dr,
            double dg, double db, double da, double sr, double sg, double sb,
            double sa, double s) {
        this.ambient = new Color(ar, ag, ab, aa);
        this.diffuse = new Color(dr, dg, db, da);
        this.specular = new Color(sr, sg, sb, sa);
        this.shininess = new Color(s, 0, 0);
    }

    /** Default Constructor */
    public Material(Color ambAndDiff, Color specular, Color s) {
        this.ambient = ambAndDiff;
        this.diffuse = ambAndDiff;
        this.specular = specular;
        this.shininess = s;
    }

    /////////////////////////////
    //    GETTERS / SETTERS    //
    /////////////////////////////
    /** @return Returns the ambient. */
    public Color getAmbient() {
        return this.ambient;
    }
    /** @param ambient The ambient to set. */
    public void setAmbient(Color ambient) {
        this.ambient = ambient;
    }
    /** @return Returns the difuse. */
    public Color getDiffuse() {
        return this.diffuse;
    }
    /** @param difuse The difuse to set. */
    public void setDiffuse(Color difuse) {
        this.diffuse = difuse;
    }
    /** @return Returns the shininess. */
    public Color getShininess() {
        return this.shininess;
    }
    /** @param shininess The shininess to set. */
    public void setShininess(Color shinies) {
        this.shininess = shinies;
    }
    /** @return Returns the specular. */
    public Color getSpecular() {
        return this.specular;
    }
    /** @param specular The specular to set. */
    public void setSpecular(Color specular) {
        this.specular = specular;
    }
}
 
