package feec.ia725.cg.core;

import javax.media.opengl.GL;

/**
 * @author Lucas Shindi Shimo (lshimo@gmail.com)
 * @author Rafael Henrique Manoel (rafael.manoel@gmail.com)
 *
 * Modelo3D com 1 material para o modelo todo e sem textura.
 */
public class SimpleModel extends AbstractModel{
 
	private int polygonMode;
	private String code;
	private Color color;
	private Material material;
	private float[][] vertexes;
    private int faces[][];
    private float[][] normals;
    private int displayList;
    private boolean displayListGenerated = false;
    
    /** BoundingBox do Model */
    private BoundingBox boundingBox;

    public SimpleModel() {
    	boundingBox = new BoundingBox();
	}
    
    /////////////////////////////
    //    GETTERS / SETTERS    //
    /////////////////////////////
    
    /**
     * Transformaes de cores
     */
    private void doColorTransformations(GL gl) {
        if (material == null) {
            material = Material.BRONZE;
        }
    	gl.glColor3fv(material.getAmbient().asArray(), 0);

        gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_AMBIENT,
        		material.getAmbient().asArray(), 0);
        gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_DIFFUSE,
        		material.getDiffuse().asArray(), 0);
        gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_SPECULAR,
        		material.getSpecular().asArray(), 0);
        gl.glMaterialfv(GL.GL_FRONT_AND_BACK, GL.GL_SHININESS,
        		material.getShininess().asArray(), 0);
    }
    
    public void generateDisplayList(GL gl) {
        if (material == null) {
            material = Material.BRONZE;
        }
        displayList = gl.glGenLists(1);
        gl.glNewList(displayList, GL.GL_COMPILE);
        	// faz a colorizao do objeto
        	doColorTransformations(gl);
        
        	// Loop para percorrer as faces triangulares e densenha-las
        	// definnindo as normais j calculadas utilizando o algoritmo de Gouraud
            gl.glBegin(GL.GL_TRIANGLES);
                for (int i = 0; i < faces.length; ++i) {
                    gl.glNormal3fv(normals[faces[i][0]], 0);
                    gl.glVertex3fv(vertexes[faces[i][0]], 0);
                    
                    gl.glNormal3fv(normals[faces[i][1]], 0);
                    gl.glVertex3fv(vertexes[faces[i][1]], 0);
                    
                    gl.glNormal3fv(normals[faces[i][2]], 0);
                    gl.glVertex3fv(vertexes[faces[i][2]], 0);
                }
            gl.glEnd();
        gl.glEndList();
    }
    
    /**
     * Seta as dimenses da boundingbox usando os maximos e minimos
     * nas coordenadas x,y,z.
     *
     */
    private void setBoundingBoxDimension() {
       	
    	for (int i = 0; i < this.vertexes.length; i++) {
    		// Coordenadas mximas
    		boundingBox.setMaxX(vertexes[i][0]);	// coord x.
    		boundingBox.setMaxY(vertexes[i][1]);	// coord y.
    		boundingBox.setMaxZ(vertexes[i][2]);	// coord z.
    		
    		// Coordenadas mnimas
    		boundingBox.setMinX(vertexes[i][0]);	// coord x.
    		boundingBox.setMinY(vertexes[i][1]);	// coord y.
    		boundingBox.setMinZ(vertexes[i][2]);	// coord z.
    	}
    	
    }
    
    /**
     * Desenha o objeto na tela.
     */
    public void render(GL gl) {
        if (!displayListGenerated) {
            displayListGenerated = true;
            generateDisplayList(gl);
        }
        
        // faz as transformaes geomtricas (translacao e rotao) do objeto
        gl.glPushMatrix();
        doGeometricTransformations(gl);
        // renderiza o objeto
        gl.glCallList(displayList);
        gl.glPopMatrix();
    }
    
    /**
     * Normaliza o modelo
     */
	public void normalize() {
		// obtendo a maior das coordenadas para normalizar
        float maxCoord = boundingBox.getMaxCoord();
        
        // normaliza a boundingbox tbm
        boundingBox.normalize();
        
        for (int i=0; i<vertexes.length; i++) {
            vertexes[i][0] = vertexes[i][0] / maxCoord;
            vertexes[i][1] = vertexes[i][1] / maxCoord;
            vertexes[i][2] = vertexes[i][2] / maxCoord;
        }
	}

	/**
	 * Aumenta/Diminui o tamanho de um objeto
	 */
	public void scale(float scale) {

        boundingBox.scale(scale);
        for (int i=0; i<vertexes.length; i++) {
            vertexes[i][0] = vertexes[i][0] * scale;
            vertexes[i][1] = vertexes[i][1] * scale;
            vertexes[i][2] = vertexes[i][2] * scale;
        }
	}
    
    /**
     * @return Returns the vertex of faces.
     */
    public int[][] getFaces() {
        return this.faces;
    }
    
    /**
     * @param faces The vertex of face indexes to set.
     */
    public void setFaces(int[][] faces) {
        this.faces = faces;
    }
    
    /**
     * @return Returns the code.
     */
    public String getCode() {
        return this.code;
    }
    /**
     * @param code The code to set.
     */
    public void setCode(String code) {
        this.code = code;
    }
    /**
     * @return Returns the color.
     */
    public Color getColor() {
        return this.color;
    }
    /**
     * @param color The color to set.
     */
    public void setColor(Color color) {
        this.color = color;
    }
    /**
     * @return Returns the material.
     */
    public Material getMaterial() {
        return this.material;
    }
    /**
     * @param material The material to set.
     */
    public void setMaterial(Material material) {
        this.material = material;
    }
    /**
     * @return Returns the polygonMode.
     */
    public int getPolygonMode() {
        return this.polygonMode;
    }
    /**
     * @param polygonMode The polygonMode to set.
     */
    public void setPolygonMode(int polygonMode) {
        this.polygonMode = polygonMode;
    }
    
    /**
     * @return Returns the vertexes.
     */
    public float[][] getVertexes() {
        return this.vertexes;
    }
    /**
     * @param vertexes The vertexes to set.
     */
    public void setVertexes(float[][] vertexes) {
        this.vertexes = vertexes;
        // configura as dimenses da bounding box
        this.setBoundingBoxDimension();
    }
    
    /**
     * @param normals The normals to set.
     */
    public void setNormals(float[][] normals) {
        this.normals = normals;
    }
    
    /**
     * @return Returns the normals.
     */
    public float[][] getNormals() {
        return this.normals;
    }

    /**
     * @return Returns the boundingBox.
     */
    public BoundingBox getBoundingBox() {
        return this.boundingBox;
    }
    
    /**
     * Verifica se a posicao v esta colidindo com o objeto
     * @param v
     * @return
     */
    public boolean hasColided(Vertex v) {
        //posicao do objeto
        Vertex p = getPosition();
        
        if (p == null) {
            return false;
        }

        // verifica se o vetor posicao v, esta dentro da boundingbox do objeto
        return v.getX() > p.getX() + boundingBox.getMinX() && v.getX() < p.getX() + boundingBox.getMaxX()
            && v.getY() > p.getY() + boundingBox.getMinY() && v.getY() < p.getY() + boundingBox.getMaxY()
            && v.getZ() > p.getZ() + boundingBox.getMinZ() && v.getZ() < p.getZ() + boundingBox.getMaxZ();
    }
    
}
 
