/*
 * TerrainModel.java
 * Created in 21/04/2006
 */
package feec.ia725.cg.core.extensions;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import feec.ia725.cg.core.SimpleModel;
import feec.ia725.cg.core.Vertex;

/**
 * @author Lucas Shindi Shimo (lshimo@gmail.com)
 *
 */
public class TerrainModel extends SimpleModel {
    private Ver[] vers = null;

    List normalsXPos = new ArrayList();
    List normalsZPos = new ArrayList();

    public void prepare() {
        float[][] vert = this.getVertexes();
        vers = new Ver[vert.length];

        for (int i = 0; i < vert.length; i++) {
            vers[i] = new Ver(vert[i], i);
        }

        // para todas as faces
        int[][] face = this.getFaces();
        for (int i = 0; i < face.length; i++) {
            vers[face[i][0]].addNear(vers[face[i][1]]);
            vers[face[i][1]].addNear(vers[face[i][2]]);
            vers[face[i][2]].addNear(vers[face[i][0]]);
        }
    }

    /** (non-Javadoc)
     * @see feec.ia725.cg.core.extensions.Terrain#getNormal(float, float)
     */
    public float[] getNormal(float[] xy) {
        Ver ver = this.getVer(xy);
        return this.getNormals()[ver.v];
    }

    public float[] getVers(float[] xy) {
        Ver ver = this.getVer(xy);
        return ver.pos;
    }

    /**
     * Pego o mais perto
     * @param xz
     * @return
     */
    private Ver getVer(float[] xz) {
        Ver ver = this.vers[0];

        boolean found = false;
        do {
            Ver near = ver.getNear(xz);

            if (near == ver) {
                found = true;
            } else {
                ver = near;
            }
        } while (!found);

        return ver;
    }

    private class Ver {
        private float pos[];
        private int v;
        private List near = new ArrayList();
        /**
         * 
         */
        public Ver(float gepos[], int ver) {
            this.pos = gepos;
            this.v = ver;
        }

        public void addNear(Ver ver) {
            this.near.add(ver);
        }
        /**
         * @see java.lang.Object#toString()
         */
        public String toString() {
            return "(" + pos[0] + "," + pos[2] + ")";
        }

        public Ver getNear(float[] xz) {
            Ver result = this;
            float dist = Vertex.distance(this.pos, xz);
            for (Iterator iter = near.iterator(); iter.hasNext();) {
                Ver ver = (Ver) iter.next();
                float d = Vertex.distance(xz, ver.pos);
                if (d < dist) {
                    result = ver;
                    dist = d;
                }
            }
            return result;
        }
    }
}
