/*
 * GameEngine.java
 * Created in 17/04/2006
 */
package feec.ia725.cg.engine;

import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;

import feec.ia725.cg.core.Angle3d;
import feec.ia725.cg.core.Color;
import feec.ia725.cg.core.Coordinate4d;
import feec.ia725.cg.core.Light;
import feec.ia725.cg.core.Material;
import feec.ia725.cg.core.SimpleModel;
import feec.ia725.cg.core.Vertex;
import feec.ia725.cg.core.extensions.TerrainModel;
import feec.ia725.cg.engine.physical.Particle;
import feec.ia725.cg.factory.ModelFactory;
import feec.ia725.cg.factory.ModelLoader;
import feec.ia725.cg.util.GameUtil;

/**
 * @author Lucas Shindi Shimo (lshimo@gmail.com)
 * @author Rafael Henrique Manoel (rafael.manoel@gmail.com)
 *
 */
public class GameEngine {
    /** Scene Engine */
    private SceneEngine sceneEngine;
    /** Input Engine */
    private InputEngine inputEngine;
    /** Sistema de movimento e coliso */
    private MovementSystem movementEngine;

    /** Fora de disparo do projetil */
    private double power = 1000;
    private static final double POWER_FACTOR = 100;
    
    /** Tempo anterior */
    private static double oldTime = 0;
    
    /** Tempo atual */
    private static double currentTime = 0;

    public static final String MISSEL1 = "tomahawk";
    public static final String MISSEL2 = "bala de canhao";
    public static final String MISSEL3 = "B-57";
    
    private float score = 0;
    
    
    private String projectilCode;

	/** Camera */
//	private Camera camera = new Camera();
    
    private double phi;
    private double theta = 0;
    private int forward;
    private int turn;

    private Camera camera;
    private Coordinate4d tankPos;

    private SimpleModel tank;
    private SimpleModel turret;
    private SimpleModel mainGun;
    private TerrainModel terrain;
    /////////////////////////////
    //    BUSINESS METHODS     //
    /////////////////////////////
    /**
     * 
     */
    public void init() {
    	float scale = 5f;
    	
        Light light = new Light();
        light.setPosition(-700, 700, 700);
        light.setDirection(0, 0, 0);
        light.setAmbient(Color.GRAY);
        light.setDifuse(Color.WHITE);
        light.setSpecular(Color.WHITE);

        camera = new Camera();
        camera.setPhi(0);
        camera.setTheta(0);
        camera.setPosition(new Vertex(0, 2, 7));
        camera.setUpVector(new Coordinate4d(0, 1, 0, 1));

        tankPos = new Coordinate4d(0, 0, 0, 1);
        ModelLoader loader = ModelFactory.getVTKLoader();

        tank = (SimpleModel) loader.load("base.vtk.norm");
        tank.setCode("tank");
        tank.setPosition(tankPos);
        tank.setAngle3d(new Angle3d(0, 0));
        tank.setMaterial(Material.CHROME);
        tank.scale(scale);

        turret = (SimpleModel) loader.load("turret.vtk.norm");
        turret.setCode("turret");
        turret.setMaterial(Material.JADE);
        turret.scale(scale);

        mainGun = (SimpleModel) loader.load("maingun.vtk.norm");
        mainGun.setCode("mainGun");
        mainGun.setMaterial(Material.BRASS);
        mainGun.scale(scale);

        terrain = (TerrainModel) loader.load("terreno3.vtk");
        terrain.setMaterial(Material.BRONZE);
        terrain.normalize();
        terrain.scale(100);
        terrain.prepare();

        // projetil 1 - Missel
        SimpleModel missel = (SimpleModel) loader.load("missil1.vtk");
        missel.setCode(MISSEL1);
        missel.setMaterial(Material.GREEN_PLASTIC);
        missel.normalize();
        missel.scale(0.8f);
        
        // projetil 2 - Missel
        SimpleModel missel2 = (SimpleModel) loader.load("missil2.vtk");
        missel2.setCode(MISSEL2);
        missel2.setMaterial(Material.RUBY);
        missel2.normalize();
        missel2.scale(0.8f);
        
        
        // projetil 3 - Missel
        SimpleModel missel3 = (SimpleModel) loader.load("missil3.vtk");
        missel3.setCode(MISSEL3);
        missel3.setMaterial(Material.CHROME);
        missel3.normalize();
        missel3.scale(0.8f);
        
        // alvo 1
        SimpleModel target = (SimpleModel) loader.load("target1.vtk");
        target.setCode("target1");
        target.normalize();
        target.scale(4);
        target.setPosition(new Vertex(-20,1, 30));
        target.setAngle3d(new Angle3d(180,0));
        target.getAngle3d().setThetaAxle(new Vertex(1,0,0));
        target.setMaterial(Material.EMERALD);
        // adiciona o modelo  cena
        sceneEngine.addModel(target);

        target = (SimpleModel) loader.load("target2.vtk");
        target.setCode("target2");
        target.normalize();
        target.scale(5);
        target.setPosition(new Vertex(30,1, -30));
        // ajustando o angulo
        target.setAngle3d(new Angle3d(180,0));
        target.getAngle3d().setThetaAxle(new Vertex(1,0,0));
        target.setMaterial(Material.BRASS);
        // adiciona o modelo  cena
        sceneEngine.addModel(target);
        
        SimpleModel house = (SimpleModel) loader.load("house.vtk");
        house.normalize();
        house.scale(100);
        house.setAngle3d(new Angle3d(180,0));
        house.getAngle3d().setThetaAxle(new Vertex(1,0,0));
        house.setPosition(new Vertex(-50,6,-60));
        house.setMaterial(Material.RUBY);
        house.setCode("house");
        
        // adiciona o modelo  cena
        sceneEngine.addModel(house);
        sceneEngine.addLight(light);
        sceneEngine.addModel(terrain);
        sceneEngine.addProjectile(missel);
        sceneEngine.addProjectile(missel2);
        sceneEngine.addProjectile(missel3);
        
               
        oldTime = System.currentTimeMillis();
    }

    /**
     * Tiro.
     * Calcula a direo, angulo e fora para o disparo e insere
     * um projetil na lista de projeteis.
     *
     */
    public void shoot() {
    
        double angT = tank.getAngle3d().getTheta() - theta;
        double PI180 = Math.PI / 180.0;

        double ly = Math.sin(phi * PI180);
        double lyXZ = Math.cos(phi * PI180);
        double lx = Math.sin(angT * PI180) * lyXZ;
        double lz = -Math.cos(angT * PI180) * lyXZ;
        Vertex velocity = new Vertex(lx, ly, lz);

        // massa da particula
        float mass = 0;
        if (projectilCode.equals(MISSEL1)) {
            mass = 25;
        } else if (projectilCode.equals(MISSEL2)) {
            mass = 10;
        } else if (projectilCode.equals(MISSEL3)) {
            mass = 50;
        }
            
            
    	Particle p = new Particle(mass, tankPos.cloneVertex());
    	p.setVelocity(velocity.multiply(power / p.getMass()));
    	p.setModelCode(projectilCode);
    	
    	// adiciona a particula na engine de movimento
    	movementEngine.addParticle(p);
    	
    }
    
    /**
     * @param drawable 
     * @see java.lang.Runnable#run()
     */
    public void run(GLAutoDrawable autoDrawable) {
    	// atualiza o sistema de fsica
    	// obtendo o delta T (segundos) para a simulao fsica.
    	currentTime = System.currentTimeMillis();
    	float dT = (float) ((currentTime - oldTime) / 1000);
    	
        // se o deltaT for zero, nao atualiza as posicoes dos objetos da cena.
    	if (dT != 0) {
    		// atualizando a posio dos objetos
        	movementEngine.operate(dT);
        	oldTime = currentTime;
        }
    	
        GL gl = autoDrawable.getGL();

        sceneEngine.render(gl);
        camera.setPosition(tankPos);

        double tankTheta = tank.getAngle3d().getTheta();
        float[] fs = tankPos.asArray();

        float[] normal = terrain.getNormal(fs);
        gl.glPushMatrix();
            gl.glTranslated(fs[0], fs[1], fs[2]);
            gl.glRotated(-tankTheta, 0f, 1f, 0f);
            gl.glRotated(Math.asin(normal[0]) * 180.0 / Math.PI, 1, 0, 0);

            tank.render(gl);
            gl.glPushMatrix();
                gl.glRotated(theta, 0f, 1f, 0f);
                turret.render(gl);
                gl.glPushMatrix();
                    gl.glTranslated(0, 1.887, -1.607);
                    gl.glRotated(phi, 1f, 0f, 0f);
                    gl.glTranslated(0, -1.887, 1.607);
                    mainGun.render(gl);
                gl.glPopMatrix();
            gl.glPopMatrix();
        gl.glPopMatrix();

        inputEngine.run();
        float walkSpeed = 1f;
        float turnSpeed = 2f;
        if (true) { // trata a rotao do tank
            tankTheta += turn * turnSpeed;
            tank.getAngle3d().setTheta(tankTheta % 360);
        }

        if (true) { // trata a rotao da camera
            camera.setTheta(-theta * 0.7 + tankTheta);
            camera.setPhi(phi * 0.5);
        }

        if (true) { // tratamento de translado
            double lx = Math.sin(tankTheta * Math.PI / 180.0);
            double lz = -Math.cos(tankTheta * Math.PI / 180.0);
            tankPos.sum(forward * walkSpeed * lx, 0, 0);
            tankPos.sum(0, 0, forward * walkSpeed * lz);
        }

        float[] y = terrain.getVers(tankPos.asArray());
        tankPos.setY(y[1]);

        GameUtil.drawAxis(gl);

        float factor = 4;
        gl.glLineWidth(3);
        gl.glBegin(GL.GL_LINES);
        gl.glColor3f(1, 1, 1);
        gl.glVertex3d(y[0], y[1], y[2]);
        gl.glVertex3d(y[0] + normal[0] * factor, y[1] + normal[1] * factor,
                y[2] + normal[2] * factor);

        gl.glColor3f(1, 1, 0);
        gl.glVertex3d(fs[0], fs[1], fs[2]);
        gl.glVertex3d(fs[0] + normal[0] * factor, fs[1] + normal[1] * factor,
                fs[2] + normal[2] * factor);
        gl.glEnd();

//        camera.getUpVector().setVertex(normal[0], normal[1], normal[2]);


        sceneEngine.displayHUD(gl);
    }

    /** Finaliza o processo. */
    public void shutDown() {
        System.out.println("Pontos = " + score);
        System.exit(0);
    }

    /////////////////////////////
    //    ATRIBUTES MODIFIES   //
    /////////////////////////////
    /**
     * Indica o sentido e mdulo do translado
     * @param value valor do sentido
     */
    public void setForward(int value) {
        this.forward = value;
    }

    /**
     * Indica a direo da converso
     * @param value valor da direo de converso
     */
    public void setTurn(int value) {
        this.turn = value;
    }

    public void updateMouseUp(double phi) {
        this.phi = phi;
    }

    public void updateMouseLeft(double theta) {
        this.theta = theta;        
    }
    /////////////////////////////
    //    GETTERS / SETTERS    //
    /////////////////////////////
    /** @return Returns the sceneEngine. */
    public SceneEngine getSceneEngine() {
        return this.sceneEngine;
    }

    /** @param sceneEngine The sceneEngine to set. */
    public void setSceneEngine(SceneEngine sceneEngine) {
        this.sceneEngine = sceneEngine;
    }

    /** @return Returns the inputEngine. */
    public InputEngine getInputEngine() {
        return this.inputEngine;
    }

    /** @param inputEngine The inputEngine to set. */
    public void setInputEngine(InputEngine inputEngine) {
        this.inputEngine = inputEngine;
    }

	public MovementSystem getMovementEngine() {
		return movementEngine;
	}

	public void setMovementEngine(MovementSystem movementEngine) {
		this.movementEngine = movementEngine;
	}
	
	public Camera getCamera() {
		return camera;
	}

    /**
     * @param projectilCode The projectilCode to set.
     */
    public void setProjectilCode(String projectilCode) {
        this.projectilCode = projectilCode;
    }
    
    /**
     * @param power The power to set.
     */
    public void increasePower() {
        this.power += POWER_FACTOR;
        System.out.println("Aumentando a fora de disparo para: " + power);
    }

    /**
     * @param power The power to set.
     */
    public void decreasePower() {
        this.power -= POWER_FACTOR;
        System.out.println("Diminuindo a fora de disparo para: " + power);
    }
    
    public void increaseScore() {
        score += 1;
    }
}
