/*
 * InputEngine.java
 * Created in 17/04/2006
 */
package feec.ia725.cg.engine;

import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;

import javax.swing.event.MouseInputListener;

import com.sun.opengl.impl.Project;


/**
 * @author Lucas Shindi Shimo (lshimo@gmail.com)
 *
 */
public class InputEngine implements KeyListener, MouseInputListener, Runnable {
    /** Engine principal */
    private GameEngine gameEngine;
    /** Indica se est movimentando */
    private int walking;
    /** Indica se est virando */
    private int turning;
    /** Indica a posio do mouse */
    private Point mouse;
    /** A diferena entre as leituras da posio X do mouse */
    private int viewX;
    /** A diferena entre as leituras da posio Y do mouse */
    private int viewY;
    /** Indica finalizao do jogo */
    private boolean escape = false;
    
    /** fator de Aumento/Diminuicao de forca de disparo */ 
    private int powerFactor;
    
    private String projectilCode = GameEngine.MISSEL1;

    /** Default Constructor */
    public InputEngine(GameEngine gameEngine) {
        this.gameEngine = gameEngine;
        gameEngine.setInputEngine(this);
    }

    /** @see java.lang.Runnable#run() */
    public void run() {
        GameEngine engine = this.getGameEngine();
        
        if (escape) {
            engine.shutDown();
        }
        
        // escolha do missel
        engine.setProjectilCode(projectilCode);
        
        // tratamento do movimento de translado
        switch (walking) {
            case 1:
            case -1:
                engine.setForward(walking);
                break;
            default:
                engine.setForward(walking);
                break;
        }

        // tratamento do movimento de converso
        switch (turning) {
            case 1:
            case -1:
                engine.setTurn(turning);
                break;
            default:
                engine.setTurn(turning);
                break;
        }

        // tramento da movimentao dos angulos
        engine.updateMouseLeft(-viewX);
        engine.updateMouseUp(-viewY);
    }

    /////////////////////////////
    //    GETTERS / SETTERS    //
    /////////////////////////////
    /** @return Returns the gameEngine. */
    public GameEngine getGameEngine() {
        return this.gameEngine;
    }

    /** @param gameEngine The gameEngine to set. */
    public void setGameEngine(GameEngine gameEngine) {
        this.gameEngine = gameEngine;
    }
    //////////////////////
    //      EVENTS      //
    //////////////////////
    /** @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent) */
    public void keyTyped(KeyEvent e) {
    }

    /** @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent) */
    public void keyPressed(KeyEvent e) {
        switch (e.getKeyCode()) {
            case KeyEvent.VK_W:
                walking = 1;
                break;
            case KeyEvent.VK_S:
                walking = -1;
                break;
            case KeyEvent.VK_D:
                turning = 1;
                break;
            case KeyEvent.VK_A:
                turning = -1;
                break;
            case KeyEvent.VK_1:
                projectilCode = GameEngine.MISSEL1;
                System.out.println("Trocando para o projtil: " + projectilCode);
                break;
            case KeyEvent.VK_2:
                projectilCode = GameEngine.MISSEL2;
                System.out.println("Trocando para o projtil: " + projectilCode);
                break;
            case KeyEvent.VK_3:
                projectilCode = GameEngine.MISSEL3;
                System.out.println("Trocando para o projtil: " + projectilCode);
                break;       
            case KeyEvent.VK_PLUS:
                getGameEngine().increasePower();
                break;
            case KeyEvent.VK_MINUS:
                getGameEngine().decreasePower();
                break;                
            case KeyEvent.VK_ESCAPE:
                escape = true;
            default:
                break;
        }
    }

    /** @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent) */
    public void keyReleased(KeyEvent e) {
        switch (e.getKeyCode()) {
            case KeyEvent.VK_W:
            case KeyEvent.VK_S:
                walking = 0;
                break;
            case KeyEvent.VK_A:
            case KeyEvent.VK_D:
                turning = 0;
            default:
                break;
        }
    }

    /** @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent) */
    public void mouseClicked(MouseEvent e) {
    	gameEngine.shoot();
    }

    /** @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent) */
    public void mousePressed(MouseEvent e) {
    }

    /** @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent) */
    public void mouseReleased(MouseEvent e) {
    }

    /** @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent) */
    public void mouseEntered(MouseEvent e) {
    }

    /** @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent) */
    public void mouseExited(MouseEvent e) {
    }

    /** @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent) */
    public void mouseDragged(MouseEvent e) {
    }

    /** @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent) */
    public void mouseMoved(MouseEvent e) {
        Point point = e.getPoint();

        if (e.isControlDown()){
            if (mouse != null) {
                viewX = (viewX + point.x - mouse.x ) % 360;
                viewY = (viewY + point.y - mouse.y ) % 360;

                // tratamento de mximos
                if (viewX > 90) {
                    viewX = 90;
                } else if (viewX < -90) {
                    viewX = -90;
                }

                if (viewY < -45) {
                    viewY = -45;
                } else if (viewY > 0){
                    viewY = 0;
                }
            }
        }
        mouse = e.getPoint();
    }
}
