package feec.ia725.cg.engine;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import feec.ia725.cg.core.SimpleModel;
import feec.ia725.cg.core.Vertex;
import feec.ia725.cg.engine.physical.Particle;


/**
 * Sistema de particulas.
 * 
 * @author Rafael
 */
public class MovementSystem {

	/** Lista de particulas do sistema */
	static List particles;
	
	/** vetor da acelerao da gravidade */
	private Vertex g = new Vertex(0.0f,-9.81f,0.0f);
    
    private GameEngine gameEngine;

	/**
	 * Inicializa
	 *
	 */
	public MovementSystem(GameEngine gameEngine) {
		particles = new Vector();
        this.gameEngine = gameEngine;
		gameEngine.setMovementEngine(this);
		
	}

	public void addParticle(Particle p) {
		particles.add(p);
	}
	
	/**
	 * Atualiza o sistema de partculas num dado intervalo de tempo
	 * @param milliseconds
	 */
	public void simulate(float milliseconds) {
		
		Enumeration elements = ((Vector) particles).elements();
		
		for (;elements.hasMoreElements();) {
			Particle p = (Particle) elements.nextElement();
			
			p.simulate(milliseconds);
			
			// tratamento de colisao
            treatColision(p);
            
			if (p.getPosition().getY() <= 0f) {
				particles.remove(p);
			}			
		}
	}

	/**
	 * Faz todas as operaes, ou seja  uma iterao completa sobre a particula:
	 * 1 - Inicializa<br>
	 * 2 - Obtm a resultante das foras que atuam sobre a partcula<br>
	 * 3 - Roda a simulaao para obter a nova posio da particula<br>
	 * @param dt
	 */
	public void operate(float dt) {
		init();
		solve();
		simulate(dt);
	}
	
	/**
	 * Soluciona o sistema, ou seja, para cada particula aplica as foras
	 * globais
	 *
	 */
	public void solve() {
		for (Iterator itParticles = particles.iterator(); itParticles.hasNext();) {
			Particle p = (Particle) itParticles.next();
			
			// encontrando a Fora que age sobre a partcula
			p.applyForce(g.multiply(p.getMass()));
		}
	}
	
	/**
	 * Inicializa todas as particulas do sistema
	 *
	 */
	private void init() {
		for (Iterator itParticles = particles.iterator(); itParticles.hasNext();) {
			Particle p = (Particle) itParticles.next();
			p.init();
		}
	}
	
    private void treatColision(Particle p) {
        p.getPosition();

        // obtem todos os objetos da cena
        Collection models = gameEngine.getSceneEngine().getModels().values();
        for (Iterator it = models.iterator(); it.hasNext();) {
            SimpleModel model = (SimpleModel) it.next();
            
            // verifica se colidiu com algum modelo
            if (model.hasColided(p.getPosition())) {
                System.out.println("O Projetil Colidiu com " + model.getCode());
                gameEngine.increaseScore();
            }
            
        }
    }
    
	/**
	 * Obtm as partculas do sistemas
	 * @return
	 */
	public List getParticles() {
		return particles;
	}
}
