package feec.ia725.cg.engine;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.media.opengl.GL;
import javax.media.opengl.glu.GLU;

import feec.ia725.cg.core.Angle3d;
import feec.ia725.cg.core.Color;
import feec.ia725.cg.core.Light;
import feec.ia725.cg.core.Model;
import feec.ia725.cg.core.SimpleModel;
import feec.ia725.cg.core.Vertex;
import feec.ia725.cg.engine.physical.Particle;

public class SceneEngine {
    private GameEngine gameEngine;

    private int glLight[] = new int[] {GL.GL_LIGHT0, GL.GL_LIGHT1, GL.GL_LIGHT2,
            GL.GL_LIGHT3, GL.GL_LIGHT4, GL.GL_LIGHT5, GL.GL_LIGHT6,
            GL.GL_LIGHT7};
    
    /** Mapa com os modelos 3D */
	private Map models = new HashMap();
    
    /** Mapa com os projteis */
    private Map projectils = new HashMap();
	 
	private List lights = new ArrayList();

	/** Cor de fundo default */
    private Color backGroundColor = Color.LIGHT_GRAY;

    /**
     * Construtor
     */
    public SceneEngine(GameEngine gameEngine) {
        this.gameEngine = gameEngine;
        gameEngine.setSceneEngine(this);
    }

    /**
     * Renderiza a scena
     * @param gl
     */
	public void render(GL gl) {
        GLU glu = new GLU();

        gl.glClearColor((float) backGroundColor.getX(), (float) backGroundColor
                .getY(), (float) backGroundColor.getZ(),
                (float) backGroundColor.getW());
        
        // limpando o buffer de cor e o de profundidade
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();
        // tratamento da window
        glu.gluPerspective(60, 800.0 / 600.0, 0.1, 10000);

        // posicionamento da cmera
        Camera cam = this.getCamera();
        Vertex position = cam.getPosition();
        Vertex upVector = cam.getUpVector();

        gl.glRotated(cam.getAngle().getPhi(), -1, 0, 0);
        // pequeno ajuste para olhar para o tanque
        gl.glRotated(10, 1, 0, 0);

        // posio da cmera em relao ao objeto
        gl.glTranslated(0, -5, -10);
        gl.glRotated(cam.getAngle().getTheta(), 0, 1, 0);
        
        // Posicionando a camera na tela.
        glu.gluLookAt(position.getX(), position.getY(), position.getZ(),
                position.getX(), position.getY(), position.getZ() - 5, upVector
                        .getX(), upVector.getY(), upVector.getZ());

        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();

        // tipo de renderizao
        gl.glShadeModel(GL.GL_SMOOTH);
        gl.glEnable(GL.GL_CULL_FACE);
        gl.glEnable(GL.GL_DEPTH_TEST);
//        gl.glEnable(GL.GL_BLEND);
//        gl.glBlendFunc(GL.GL_SOURCE1_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
//        gl.glBlendEquation(GL.GL_FUNC_ADD);

        gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_POLYGON);
        gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_LINE);

        // tratamento das luzes
//        setLightsOn(gl);

        // Loop para renderizar todos os objetos da cena
        Collection modelsCol = this.getModels().values();
        for (Iterator iter = modelsCol.iterator(); iter.hasNext();) {
            Model model = (Model) iter.next();
            model.render(gl);
        }

        // renderiza os projteis na tela
        renderProjectils(gl);
	}

	/**
	 * Carrega e habilita as luzes do ambiente.
	 *
	 */
	private void setLightsOn(GL gl) {
        gl.glEnable(GL.GL_LIGHTING);
        gl.glLightModelf(GL.GL_LIGHT_MODEL_LOCAL_VIEWER, GL.GL_TRUE);

        Object[] objLight = this.getLights().toArray();
        for (int i = 0; i < objLight.length && i < glLight.length; i++) {
            Light light = (Light) objLight[i];

            gl.glEnable(glLight[i]);
            gl.glLightfv(glLight[i], GL.GL_POSITION, light.getPosition()
                    .asArray(), 0);
            gl.glLightfv(glLight[i], GL.GL_SPOT_DIRECTION, light.getDirection()
                    .asArray(), 0);
            gl.glLightfv(glLight[i], GL.GL_AMBIENT, light.getAmbient()
                    .asArray(), 0);
            gl.glLightfv(glLight[i], GL.GL_DIFFUSE,
                    light.getDifuse().asArray(), 0);
            gl.glLightfv(glLight[i], GL.GL_SPECULAR, light.getSpecular()
                    .asArray(), 0);
        }		
	}
	
	/**
	 * Percorre a lista de prjeteis e os desenha na tela.
	 * @param gl
	 */
	private void renderProjectils(GL gl) {
        // iterando as particulas do sistema de fsica
        // e renderizando os projteis na posicao de cada partcula
        Vector particles = (Vector) this.gameEngine.getMovementEngine()
        	.getParticles();
        
        for (Enumeration e = particles.elements(); e.hasMoreElements();) {
        	Particle p = (Particle) e.nextElement();
        	
        	SimpleModel missel = (SimpleModel) getProjectils().get(p.getModelCode());
        	missel.setPosition(p.getPosition());
        	
        	
        	Angle3d a = new Angle3d(p.getVelocity().getTheta(),
        			p.getVelocity().getPhi(),
        			p.getVelocity().crossProduct(new Vertex(0,0,1)));	// eixo de rotao para o angulo theta
        	
        	missel.setAngle3d(a);
        	missel.render(gl);
        }
	}
	
	/**
	 * Renderiza o HUD na tela
	 * @param gl
	 */
    public void displayHUD(GL gl) {
        GLU glu = new GLU();
        gl.glMatrixMode(GL.GL_PROJECTION);
        // imforaes do HUD
        gl.glPushMatrix();
            gl.glLoadIdentity();
            glu.gluOrtho2D(0, 640, 0, 480);
            gl.glViewport(0, 0, 640, 480);

            gl.glDisable(GL.GL_CULL_FACE);
            gl.glDisable(GL.GL_DEPTH_TEST);
            gl.glDisable(GL.GL_LIGHTING);

            gl.glMatrixMode(GL.GL_MODELVIEW);
            // crosshair
            gl.glPushMatrix();
                gl.glLoadIdentity();
                gl.glColor4f(0f, 0f, 1f, 0.5f);
                gl.glLineWidth(1);
                gl.glBegin(GL.GL_LINES);
                    gl.glVertex2d(315, 240);
                    gl.glVertex2d(325, 240);
                    gl.glVertex2d(320, 235);
                    gl.glVertex2d(320, 245);
                gl.glEnd();
            gl.glPopMatrix();

            gl.glPushMatrix();
                gl.glLoadIdentity();
                gl.glColor4f(1f, 0f, 0f, 0.5f);
                gl.glBegin(GL.GL_QUADS);
                    gl.glVertex2d(0,0);
                    gl.glVertex2d(0,100);
                    gl.glVertex2d(100,100);
                    gl.glVertex2d(100,0);
                gl.glEnd();
            gl.glPopMatrix();
        gl.glPopMatrix();
    }
    /////////////////////////////
    //    ATRIBUTES MODIFIES   //
    /////////////////////////////
    public void addLight(Light light) {
        lights.add(light);
    }

    public void addModel(Model model) {
        models.put(model.getCode(), model);
    }
    
    public void getModel(String code) {
        this.getModels().get(code);
    }
    
    /**
     * Adiciona um projetil
     * @param model
     */
    public void addProjectile(Model model) {
        projectils.put(model.getCode(), model);
    }
    
    /**
     * @return Returns the projectils.
     */
    public Map getProjectils() {
        return this.projectils;
    }
    
    /////////////////////////////
    //    GETTERS / SETTERS    //
    /////////////////////////////
    /** @return Returns the gameEngine. */
    public GameEngine getGameEngine() {
        return this.gameEngine;
    }

    /** @param gameEngine The gameEngine to set. */
    public void setGameEngine(GameEngine gameEngine) {
        this.gameEngine = gameEngine;
    }

    /**
     * @return Returns the camera.
     */
    public Camera getCamera() {
        return gameEngine.getCamera();
    }
//    /**
//     * @param camera The camera to set.
//     */
//    public void setCamera(Camera camera) {
//        this.camera = camera;
//    }
    /**
     * @return Returns the lights.
     */
    public List getLights() {
        return this.lights;
    }
    /**
     * @param lights The lights to set.
     */
    public void setLights(List lights) {
        this.lights = lights;
    }
    /**
     * @return Returns the models.
     */
    public Map getModels() {
        return this.models;
    }
    /**
     * @param models The models to set.
     */
    public void setModels(Map models) {
        this.models = models;
    }
}
 
