package feec.ia725.cg.engine.physical;

import feec.ia725.cg.core.Vertex;

/**
 * Simula uma partcula, um ponto material.
 * @author Rafael
 *
 */
public class Particle {

	/** posio da partcula */
	private Vertex position;

	/** vetor velocidade */
	private Vertex velocity;

	/** Vetor resultante das foras que agem sobre a partcula */
	private Vertex force;

	/** massa da partcula */
	private float mass;
	
	/** Identificador da Particula */
	private String modelCode;
	
	private boolean live=true;

	/**
	 * @return Returns the live.
	 */
	public boolean isLive() {
		return live;
	}

	/**
	 * @param live The live to set.
	 */
	public void setLive(boolean live) {
		this.live = live;
	}

	/**
	 * Construtor
	 * @param particleMass
	 * @param particlePosition
	 */
	public Particle(float particleMass, Vertex particlePosition) {
		this.mass = particleMass;
		this.position = particlePosition;
		this.velocity = new Vertex(0,0,0);
	}
	
	/**
	 * Inicializa os atributos da partcula
	 *
	 */
	public void init() {
		// zera a resultante das foras
		this.force = new Vertex(0,0,0);
	}

	/**
	 * Aplica uma fora sobre a partcula
	 * @param force
	 * 	Fora a ser aplicada
	 */
	public void applyForce(Vertex newForce) {
		force.sum(newForce);
	}

	/**
	 * Simula a fsica da partcula dado um intervalo de tempo
	 * @param dt
	 * 	Intervalo de tempo
	 */
	public void simulate(float dt) {
		// vel += (force / m) * dt
		velocity.sum(force.multiply(dt / mass));

		// encontrando a nova posicao da particula
		position.sum(velocity.multiply(dt));
	}
	
	/**
	 * @return Returns the position.
	 */
	public Vertex getPosition() {
		return position;
	}

	/**
	 * @param position The position to set.
	 */
	public void setPosition(Vertex position) {
		this.position = position;
	}

	/**
	 * 
	 * @param velocity
	 */
	public void setVelocity(Vertex velocity) {
		this.velocity = velocity;
	}
	
	/**
	 * Retorna a massa da particula
	 * @return
	 */
	public float getMass() {
		return mass;
	}

	/**
	 * @return Returns the velocity.
	 */
	public Vertex getVelocity() {
		return velocity;
	}

	/**
	 * @return Returns the modelId.
	 */
	public String getModelCode() {
		return modelCode;
	}

	/**
	 * @param modelId The modelId to set.
	 */
	public void setModelCode(String modelCode) {
		this.modelCode = modelCode;
	}

	
}
