/*
 * OJBLoader.java Created in 17/04/2006
 */
package feec.ia725.cg.factory;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import feec.ia725.cg.core.Color;
import feec.ia725.cg.core.CompositeModel;
import feec.ia725.cg.core.Material;
import feec.ia725.cg.core.Model;

/**
 * @author Lucas Shindi Shimo (lshimo@gmail.com)
 */
public class OBJLoader implements ModelLoader {
    private static final int VERTEX_NORMAL = 1;
    private static final int VERTEX = 2;
    private static final int FACE = 3;
    private static final int MATERIAL = 4;
    private static final int MATERIAL_FILE = 5;
    private static final int AMBIENT_COMPONENT = 5;
    private static final int DIFFUSE_COMPONENT = 5;
    private static final int SPECULAR_COMPONENT = 5;

    private CompositeModel models = new CompositeModel();

    private static OBJLoader instance;

    private static Map map;
    
    private ArrayList vertices = new ArrayList();
    private ArrayList normals = new ArrayList();
    private Map materials = new HashMap();

    /**
     * Private Constructor
     */
    private OBJLoader() {
        map = new HashMap();

        map.put("vn", new Integer(VERTEX_NORMAL));
        map.put("v", new Integer(VERTEX));
        map.put("f", new Integer(FACE));
        map.put("newmtl", new Integer(MATERIAL));
        map.put("mtllib", new Integer(MATERIAL_FILE));
        map.put("Ka", new Integer(AMBIENT_COMPONENT));
        map.put("Kd", new Integer(DIFFUSE_COMPONENT));
        map.put("Ks", new Integer(SPECULAR_COMPONENT));
    }

    /**
     * @see feec.ia725.cg.factory.ModelLoader#load(java.lang.String)
     */
    public synchronized Model load(String fileName) {
        BufferedReader reader;
        String[] line;

        InputStream is = Thread.currentThread().getContextClassLoader()
                .getResourceAsStream(fileName);

        if (is == null) {
            throw new RuntimeException("No foi possvel encontrar o arquivo: "
                    + fileName);
        }

        reader = new BufferedReader(new InputStreamReader(is));

        try {
            do {
                line = reader.readLine().split(" ");
    
                Integer key = (Integer) map.get(line[0]);
                switch (key.intValue()) {
                    case MATERIAL_FILE:
                        getMaterialsFromFile(line[1]);
                        break;
                        
                    case VERTEX:
                        break;
                        
                    case VERTEX_NORMAL:
                        break;
                        
                    case FACE:
                        break;
    
                    default:
                        break;
                }
            } while (line != null);
        } catch (IOException e) {
            throw new RuntimeException(
                    "Erro ao analisar o arquivo do modelo OBJ", e);
        }
        return models;
    }
    
    public void getMaterialsFromFile(String fileName) {
        BufferedReader reader;
        String[] line;
        String materialName;
        Color tempColor;
        Material tempMaterial;

        InputStream is = Thread.currentThread().getContextClassLoader()
                .getResourceAsStream(fileName);

        if (is == null) {
            throw new RuntimeException("No foi possvel encontrar o arquivo: "
                    + fileName);
        }

        reader = new BufferedReader(new InputStreamReader(is));
        
        do {
            do {
                try {
                    line = reader.readLine().split(" ");
                } catch (IOException e) {
                    throw new RuntimeException("Erro na leitura do arquivo de materiais do modelo OBJ", e);
                }
            } while ((!"newmtl".equals(line[0])) && (line != null));
            
            if (line == null) {
                throw new RuntimeException("Erro na leitura do arquivo de materiais do modelo OBJ");
            }
            
            materialName = line[1];
            
            try {
                line = reader.readLine().split(" ");
                
            } catch (IOException e) {
                throw new RuntimeException("Erro na leitura do arquivo de materiais do modelo OBJ", e);
            }
        } while (line != null);
    }

    /**
     * @return Sigleton do loader
     */
    static synchronized ModelLoader getInstance() {
        if (instance == null) {
            instance = new OBJLoader();
        }
        return instance;
    }
}
