/*
 * VTKLoader.java Created in 17/04/2006
 */
package feec.ia725.cg.factory;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import feec.ia725.cg.core.Model;
import feec.ia725.cg.core.SimpleModel;
import feec.ia725.cg.core.Vertex;
import feec.ia725.cg.core.extensions.TerrainModel;

/**
 * @author Lucas Shindi Shimo (lshimo@gmail.com)
 */
public class VTKLoader implements ModelLoader {
    private SimpleModel model;

    private static VTKLoader instance;

    /**
     * Private Constructor
     */
    private VTKLoader() {
    }

    /**
     * @see feec.ia725.cg.factory.ModelLoader#load(java.lang.String)
     */
    public synchronized Model load(String fileName) {
//        model = new SimpleModel();
        model = new TerrainModel();

        BufferedReader reader;

        InputStream is = Thread.currentThread().getContextClassLoader()
                .getResourceAsStream(fileName);

        if (is == null) {
            throw new RuntimeException("No foi possvel encontrar o arquivo: "
                    + fileName);
        }

        reader = new BufferedReader(new InputStreamReader(is));

        model.setVertexes(getVertexesFromFile(reader));

        model.setFaces(getFacesFromFile(reader));

        model.setNormals(computeNormals(model.getFaces().length, model
                .getVertexes().length));

        return model;
    }

    /**
     * Mtodo que l as informaes de vrtices de um arquivo VTK
     * 
     * @param reader
     *            Stream de onde sero tiradas as informaes.
     * @return Retorna o array de vrtices lido.
     */

    private float[][] getVertexesFromFile(BufferedReader reader) {
        float vertexArray[][];
        String line[];
        int i, numberOfVertexes;

        try {
            line = reader.readLine().split(" ");

            while ((!"POINTS".equals(line[0])) && (line != null)) {
                if (line == null) {
                    throw new RuntimeException(
                            "Erro na leitura dos vertices do arquivo VTK");
                }
                line = reader.readLine().split(" ");
            }

            numberOfVertexes = Integer.parseInt(line[1]);
            vertexArray = new float[numberOfVertexes][3];
            for (i = 0; i < numberOfVertexes; ++i) {
                line = reader.readLine().split(" ");
                if (line.length != 3) {
                    throw new RuntimeException(
                            "Erro na leitura dos vertices do arquivo VTK");
                }
                vertexArray[i][0] = Float.parseFloat(line[0]);
                vertexArray[i][1] = Float.parseFloat(line[1]);
                vertexArray[i][2] = Float.parseFloat(line[2]);
            }
        } catch (IOException e) {
            throw new RuntimeException(
                    "Erro na leitura das informacoes de vertices do arquivo VTK",
                    e);
        }

        return vertexArray;
    }

    /**
     * Mtodo
     * 
     * @param reader
     * @return
     */
    private int[][] getFacesFromFile(BufferedReader reader) {
        int facesArray[][];
        String line[];
        int i, numberOfFaces;

        try {
            line = reader.readLine().split(" ");

            while ((!"POLYGONS".equals(line[0])) && (line != null)) {
                if (line == null) {
                    throw new RuntimeException(
                            "Erro na leitura das faces do arquivo VTK");
                }
                line = reader.readLine().split(" ");
            }

            numberOfFaces = Integer.parseInt(line[1]);
            facesArray = new int[numberOfFaces][3];

            for (i = 0; i < numberOfFaces; ++i) {
                line = reader.readLine().split(" ");
                if (line.length != 4) {
                    throw new RuntimeException(
                            "Erro na leitura das faces do arquivo VTK");
                }
                facesArray[i][0] = Integer.parseInt(line[1]);
                facesArray[i][1] = Integer.parseInt(line[2]);
                facesArray[i][2] = Integer.parseInt(line[3]);
            }
        } catch (IOException e) {
            throw new RuntimeException(
                    "Erro na leitura das informacoes de faces do arquivo VTK",
                    e);
        }

        return facesArray;
    }

    private float[][] computeNormals(int numberOfFaces, int numberOfVertexes) {
        int i;
        float tempNormals[][];
        float faceNormal[], tempVector1[], tempVector2[];

        tempNormals = new float[numberOfVertexes][3];

        float[][] vertexes = model.getVertexes();
        int[][] faces = model.getFaces();

        for (i = 0; i < numberOfFaces; ++i) {
            tempVector1 = Vertex.diff(vertexes[faces[i][1]],
                    vertexes[faces[i][0]]);
            tempVector2 = Vertex.diff(vertexes[faces[i][2]],
                    vertexes[faces[i][0]]);
            faceNormal = Vertex.crossProduct(tempVector1, tempVector2);
            Vertex.normalizeVector(faceNormal);

            tempNormals[faces[i][0]][0] += faceNormal[0];
            tempNormals[faces[i][0]][1] += faceNormal[1];
            tempNormals[faces[i][0]][2] += faceNormal[2];

            tempNormals[faces[i][1]][0] += faceNormal[0];
            tempNormals[faces[i][1]][1] += faceNormal[1];
            tempNormals[faces[i][1]][2] += faceNormal[2];

            tempNormals[faces[i][2]][0] += faceNormal[0];
            tempNormals[faces[i][2]][1] += faceNormal[1];
            tempNormals[faces[i][2]][2] += faceNormal[2];
        }

        for (i = 0; i < numberOfVertexes; ++i) {
            Vertex.normalizeVector(tempNormals[i]);
        }

        return tempNormals;
    }

    /**
     * @return Sigleton do loader
     */
    static synchronized ModelLoader getInstance() {
        if (instance == null) {
            instance = new VTKLoader();
        }
        return instance;
    }
}
